'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _jwtDecode = require('jwt-decode');

var _jwtDecode2 = _interopRequireDefault(_jwtDecode);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// TODO check and warn if regenerator-runtime not installed / present

var ApolloPassport = function () {
  function ApolloPassport(_ref) {
    var _this = this;

    var apolloClient = _ref.apolloClient;

    _classCallCheck(this, ApolloPassport);

    this.apolloClient = apolloClient;

    this._subscribers = new Set();
    this.strategies = {};

    // Redux support.  This gets set this apolloClient.middleware().
    this.store = null;

    this._token = localStorage.getItem('apToken');

    // Set initial values
    this.setState(true);

    if (this._token) {
      // constructor is a synchronous method, queue this for after.
      setTimeout(function () {
        _this.assertToken();
      }, 1);
    }
  }

  _createClass(ApolloPassport, [{
    key: 'assertToken',
    value: function assertToken() {}

    /* modules */

  }, {
    key: 'use',
    value: function use(strategyName, Strategy) {
      this.strategies[strategyName] = new Strategy(this);
    }
  }, {
    key: 'extendWith',
    value: function extendWith(obj) {
      _lodash2.default.extend(this, obj);
    }

    /* actions */

  }, {
    key: 'loginStart',
    value: function loginStart() {}
  }, {
    key: 'loginComplete',
    value: function loginComplete(result, dataKey) {
      if (result.errors) {
        console.error("A server side error was thrown during the Apollo Passport GraphQL query:");
        console.error(result.errors);
        return;
        // should we still update state?
      }

      var queryResult = result.data[dataKey];
      var data = queryResult.token ? (0, _jwtDecode2.default)(queryResult.token) : {};

      localStorage.setItem('apToken', queryResult.token);
      this.setState({
        data: data,
        verified: !!queryResult.token,
        error: queryResult.error || null
      });
    }
  }, {
    key: 'logout',
    value: function logout() {
      localStorage.removeItem('apToken');
      this.setState({ data: {}, verified: false, error: null });
    }

    /* state */

  }, {
    key: 'stateHash',
    value: function stateHash(state) {
      return JSON.stringify(state || this._state);
    }
  }, {
    key: 'setState',
    value: function setState(nextState) {
      if (!this._state) {
        this._state = {
          data: this._token ? (0, _jwtDecode2.default)(this._token) : {},
          verified: false,
          error: null
        };
        this._stateHash = this.stateHash();
      }

      // Just set initial values and exit, used in constructor
      if (nextState === true) return;

      var nextStateHash = this.stateHash(nextState);
      var hasChanged = this._stateHash !== nextStateHash;

      if (hasChanged) {
        this._state = nextState;
        this._stateHash = nextStateHash;

        // XXX todo debounce
        this.emitState();
      }
    }
  }, {
    key: 'getState',
    value: function getState() {
      return this._state;
    }
  }, {
    key: 'subscribe',
    value: function subscribe(callback) {
      this._subscribers.add(callback);
    }
  }, {
    key: 'unsubscribe',
    value: function unsubscribe(callback) {
      this._subscribers.delete(callback);
    }
  }, {
    key: 'emitState',
    value: function emitState() {
      var state = this.getState();
      this._subscribers.forEach(function (callback) {
        return callback(state);
      });
    }

    /* optional redux support */

  }, {
    key: 'reducer',
    value: function reducer() {
      var self = this;

      return function apolloPassportReducer(state, action) {
        if (action.type === 'APOLLO_PASSPORT_UPDATE') return action.state;

        // I guess this is anti-pattern in Redux but I like to populate the
        // initial value without a dispatch.
        return state || self.getState();
      };
    }
  }, {
    key: 'middleware',
    value: function middleware() {
      var self = this;
      return function apolloPassportMiddleware(store) {
        self.store = store;

        self.subscribe(function (state) {
          self.store.dispatch({
            type: 'APOLLO_PASSPORT_UPDATE',
            state: state
          });
        });

        // Don't do anything special, we just use middleware to get our store
        // with the same API as Apollo.
        return function (next) {
          return function (action) {
            return next(action);
          };
        };
      };
    }
  }]);

  return ApolloPassport;
}();

exports.default = ApolloPassport;