'use strict';

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

require('regenerator-runtime/runtime');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var token = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWV9.TJVA95OrM7E2cBab30RMHrHDcEfxjoYZgeFONFh7HgQ';
var decodedToken = {
  "sub": "1234567890",
  "name": "John Doe",
  "admin": true
};

global.window = global;
var localStorage = window.localStorage = {
  items: {},
  getItem: function getItem(name) {
    return localStorage.items[name];
  },
  setItem: function setItem(name, value) {
    return localStorage.items[name] = value;
  },
  removeItem: function removeItem(name) {
    return delete localStorage.items[name];
  }
};

var should = _chai2.default.should();

describe('ApolloPassport - client', function () {

  describe('- constructor', function () {

    it('instantiates correctly', function () {
      var apolloClient = {};
      var ap = new _index2.default({ apolloClient: apolloClient });

      ap.apolloClient.should.equal(apolloClient);
    });

    it('runs assertToken if a token exists in localStorage', function (done) {
      localStorage.setItem('apToken', token);
      var ap = new _index2.default({});
      ap.assertToken = function () {
        ap._token.should.equal(token);
        done();
      };
    });
  });

  describe('- modules', function () {

    it('can be used', function () {
      var ap = new _index2.default({});
      var TestStrategy = function TestStrategy() {};
      ap.use('test', TestStrategy);
      ap.strategies.test.should.be.an.instanceof(TestStrategy);
    });

    it('can extend the instance', function () {
      var ap = new _index2.default({});
      var methods = { a: {} };
      ap.extendWith(methods);
      ap.a.should.equal(methods.a);
    });
  });

  describe('- actions -', function () {

    it('loginStart()', function () {});

    describe('loginComplete()', function () {

      it('handles errors', function () {
        var ap = new _index2.default({});
        var result = {
          errors: []
        };

        ap.loginComplete(result, 'unused');
        // Nothing to test for for now.
      });

      it('sets state', function () {
        var ap = new _index2.default({});
        var result = {
          data: {
            passportLoginEmail: { token: token }
          }
        };

        ap.loginComplete(result, 'passportLoginEmail');
        ap.getState().should.deep.equal({
          data: decodedToken,
          verified: true,
          error: null
        });
      });
    });

    it('logout() clears state & local storage', function () {
      var ap = new _index2.default({});

      localStorage.setItem('apToken', 'a');
      ap.setState({ data: decodedToken, verified: true, error: null });

      ap.logout();

      var state = ap.getState();
      state.data.should.deep.equal({});
      state.verified.should.be.false;
    });
  });

  describe('- state', function () {

    it('allows removal subscribers', function () {
      var ap = new _index2.default({});
      var test = function test() {};

      ap.subscribe(test);
      ap.unsubscribe(test);
      ap._subscribers.has(test).should.be.false;
    });

    // subscribe+emitState tested in redux suite below.
  });

  describe('- optional redux support', function () {

    describe('- reducer', function () {

      var ap = new _index2.default({});
      var reducer = ap.reducer();
      var state = {};

      it('changes state on APOLLO_PASSPORT_UPDATE', function () {
        reducer({}, {
          type: 'APOLLO_PASSPORT_UPDATE',
          state: state
        }).should.equal(state);
      });

      it('should return given state on non matching action', function () {
        reducer(state, {}).should.equal(state);
      });

      it('should return getState as initial value', function () {
        reducer(undefined, {}).should.equal(ap.getState());
      });
    });

    describe('- middleware', function () {

      it('keeps ref to store and returns identity middleware', function () {

        var ap = new _index2.default({});
        var middleware = ap.middleware();
        var store = {};
        var m = middleware(store);
        ap.store.should.equal(store);

        // test the identity middleware
        var action = {};
        m(function () {
          return action.test = 1;
        })(action);
        action.test.should.equal(1);
      });

      it('updates redux state on apolloPassport state changes', function (done) {

        var ap = new _index2.default({});
        var middleware = ap.middleware();
        var nextState = { data: {}, verified: true, error: null };
        var store = {
          dispatch: function dispatch(action) {
            action.type.should.equal('APOLLO_PASSPORT_UPDATE');
            action.state.should.equal(nextState);
            done();
          }
        };

        middleware(store);
        ap.setState(nextState);
      });
    });
  });
});