'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path2 = require('path');

var _path3 = _interopRequireDefault(_path2);

var _passport = require('passport');

var _passport2 = _interopRequireDefault(_passport);

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

var _graphqlMerge = require('./utils/graphql-merge');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Some super weird babel thing going on here, where path is not defined.
var _path = _path3.default;

// Non-standard, see
// http://www.iana.org/assignments/jwt/jwt.xhtml
var defaultMapUserToJWTProps = function defaultMapUserToJWTProps(user) {
  return { userId: user.id };
};

function defaultCreateTokenFromUser(user) {
  return _jsonwebtoken2.default.sign(this.mapUserToJWTProps(user), this.jwtSecret, { expiresIn: "7 days" });
}

var ApolloPassport = function () {
  function ApolloPassport() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, ApolloPassport);

    // TODO actually let's rather default to a DB request on each query if no jwt secret specified
    var jwtSecret = options.jwtSecret || process.env.JWT_SECRET;
    if (!jwtSecret && !options.createTokenFromUser) throw new Error("no jwtSecret nor a custom createTokenFromUser specified");

    this._useJwt = true;

    this.strategies = {};
    this._winston = options.winston || require('winston');
    this._resolvers = require('./resolvers').default;
    this._schema = require('./schema').default;
    this.passport = _passport2.default;

    this.dbName = options.db[0];
    var userTableName = options.userTableName || 'users';

    var DBDriver = require('./db/' + this.dbName).default;
    this.db = new DBDriver(options.db[1], userTableName);

    this.jwtSecret = jwtSecret;
    this.mapUserToJWTProps = options.mapUserToJWTProps || defaultMapUserToJWTProps;
    this.createTokenFromUser = options.createTokenFromUser || defaultCreateTokenFromUser;
  }

  _createClass(ApolloPassport, [{
    key: 'use',
    value: function use(name, Strategy, options, verify) {
      if (!verify) {
        verify = options;
        options = null;
      }

      if (!options) options = this.require(name, 'defaultOptions');
      if (!verify) verify = this.dbRequire(name, 'verify');

      var instance = new Strategy(options, verify.bind(this));
      _passport2.default.use(instance);
      //this._strategies.set(instance.name, instance);

      var apWrapper = this.require(name, 'index');
      this.strategies[name] = new apWrapper(this);

      var resolvers = this.require(name, 'resolvers');
      this._resolvers = (0, _graphqlMerge.mergeResolvers)(this._resolvers, resolvers);

      var schema = this.require(name, 'schema');
      this._schema = [(0, _graphqlMerge.mergeSchemas)([this._schema[0], schema[0]])];

      // would also be nice to have a root query for available strategies
      // that could be used by the UI, and whether they are configured.
    }
  }, {
    key: 'extendWith',
    value: function extendWith(obj) {
      _lodash2.default.extend(this, obj);
    }

    /* resolve, require */

  }, {
    key: 'resolve',
    value: function resolve(module) {
      try {
        return require.resolve(module);
      } catch (err) {
        if (err.message === 'Cannot find module \'' + module + '\'') {

          /*
           * node resolves symlinks so that packages from npm link won't find
           * their own modules.  https://github.com/npm/npm/issues/5875
           * This is a small hack to make dev work easier.
           */
          if (module.charAt(0) !== '.') {
            var relative = _path.join(__dirname, '..', '..', module);
            try {
              return require.resolve(relative);
            } catch (err) {
              if (err.message !== 'Cannot find module \'' + relative + '\'') throw err;
              return null;
            }
          }

          return null;
        }
        throw new err();
      }
    }
  }, {
    key: 'require',
    value: function (_require) {
      function require(_x, _x2) {
        return _require.apply(this, arguments);
      }

      require.toString = function () {
        return _require.toString();
      };

      return require;
    }(function (strategy, module) {
      // No static analysis, but that's ok for server side
      var fromPackage = 'apollo-passport-' + strategy + '/lib/' + module;

      var resolved = this.resolve(fromPackage);
      if (!resolved) resolved = this.resolve('./strategies/' + strategy + '/' + module);
      if (!resolved) throw new Error('Cannot find \'' + fromPackage + '\'.  ' + ('Try \'npm i --save apollo-passport-' + strategy + '\'.'));

      var loaded = require(resolved);
      return loaded.__esModule ? loaded.default : loaded;
    })
  }, {
    key: 'dbRequire',
    value: function dbRequire(strategy, module) {
      return this.require(strategy, 'db/' + this.dbName + '/' + module);
    }

    /* graphql, apollo */

  }, {
    key: 'schema',
    value: function schema() {
      return this._schema;
    }
  }, {
    key: 'resolvers',
    value: function resolvers() {
      return this._bindRootMutations(this._resolvers);
    }
  }, {
    key: 'expressMiddleware',
    value: function expressMiddleware() {
      var path = arguments.length <= 0 || arguments[0] === undefined ? '/ap-auth' : arguments[0];

      return function ApolloPassportExpressMiddleware(req, res, next) {
        console.log('!!!', req.url);
      };
    }
  }, {
    key: 'wrapOptions',
    value: function wrapOptions(options) {
      var self = this;

      return function _callee(req) {
        var token, decoded;
        return regeneratorRuntime.async(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (req.headers.authorization) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', options);

              case 2:
                token = req.headers.authorization.substr(0, 7) === 'Bearer ' && req.headers.authorization.substr(7);

                if (token) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt('return', options);

              case 5:

                console.log('token', token);

                decoded = void 0;
                _context.prev = 7;

                decoded = _jsonwebtoken2.default.verify(token, self.jwtSecret);
                console.log('decoded', decoded);
                _context.next = 16;
                break;

              case 12:
                _context.prev = 12;
                _context.t0 = _context['catch'](7);

                // This could be any number of reasons but in short: user not authed.
                // Maybe pass the error to context so we can send back to client.
                // JsonWebTokenError: invalid token
                // TokenExpiredError: jwt expired
                console.log('err', _context.t0);
                return _context.abrupt('return', options);

              case 16:
                return _context.abrupt('return', _extends({}, options, {
                  context: { jwt: decoded }
                }));

              case 17:
              case 'end':
                return _context.stop();
            }
          }
        }, null, this, [[7, 12]]);
      };
    }
  }, {
    key: '_bindRootMutations',
    value: function _bindRootMutations(obj) {
      var out = {};
      for (var key in obj) {
        if (key === 'RootMutation') {
          out.RootMutation = {};
          for (var key2 in obj.RootMutation) {
            out.RootMutation[key2] = obj.RootMutation[key2].bind(this);
          }
        } else {
          out[key] = obj[key];
        }
      }return out;
    }
  }]);

  return ApolloPassport;
}();

exports.default = ApolloPassport;