'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

require('regenerator-runtime/runtime');

var _rethinkdbdash = require('../../../../db/rethinkdbdash');

var _rethinkdbdash2 = _interopRequireDefault(_rethinkdbdash);

var _rethinkdbdashInstance = require('../../../../db/rethinkdbdash-instance');

var _rethinkdbdashInstance2 = _interopRequireDefault(_rethinkdbdashInstance);

var _verify2 = require('./verify');

var _verify3 = _interopRequireDefault(_verify2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = _chai2.default.should();

describe('Strategies - local - db - rethinkdbdash - verify', function () {

  var context = {};
  var verify = _verify3.default.bind(context);
  var db = context.db = new _rethinkdbdash2.default(_rethinkdbdashInstance2.default, "users");

  var error = new Error('test error');
  context.error = null;

  context.comparePassword = function (pass1, pass2, cb) {
    if (context.error) cb(context.error);else cb(null, pass1 === pass2);
  };

  before(function _callee() {
    return regeneratorRuntime.async(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return regeneratorRuntime.awrap((0, _rethinkdbdashInstance.freshUserTable)());

          case 2:
            _context.next = 4;
            return regeneratorRuntime.awrap(db.users.insert([{
              id: "user1",
              emails: [{ address: "test@test.com" }],
              password: "test123"
            }]).run());

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, null, undefined);
  });

  it('should catch rethinkdb errors and pass to callback', function (done) {

    // Super simple API that satisfied the query in verify.js but rejects.
    context.db = {
      filter: function filter() {
        return context.db;
      },
      limit: function limit() {
        return context.db;
      },
      r: {
        row: function row() {
          return context.db;
        }
      },
      contains: function contains() {
        return context.db.users;
      },
      run: function run() {
        return new Promise(function (resolve, reject) {
          reject(error);
        });
      }
    };
    context.db.users = context.db;

    verify("foo", "bar", function (err, user) {
      err.should.equal(error);
      should.equal(user, undefined);
      context.db = db; // reset for next test
      done();
    });
  });

  it('should fail on no matching email', function (done) {

    verify("non-existing-email", "test123", function (err, user) {
      should.equal(user, false /*, "Invalid email" */);
      done();
    });
  });

  it('should fail on no matching password', function (done) {

    verify("test@test.com", "non-matching-password", function (err, user) {
      should.equal(user, false /*, "Invalid password" */);
      done();
    });
  });

  it('should call cb(err) on a thrown error', function (done) {

    context.error = error;
    verify("test@test.com", "test123", function (err, user) {
      err.should.equal(error);
      should.equal(user, undefined);
      context.error = null; // reset for next test
      done();
    });
  });

  it('return a matching user with verified password', function (done) {

    verify("test@test.com", "test123", function (err, user) {
      user.id.should.equal("user1");
      done();
    });
  });
});